@@ -30,11 +30,6 @@ class Nextline:
         # e.g., "<Nextline 'running'>"
         return f"<{self.__class__.__name__} {self.global_state!r}>"
 
-    @property
-    def global_state(self) -> str:
-        """state, e.g., "initialized", "running", "exited", "finished", "closed" """
-        return self.machine.state_name
-
     def run(self):
         """execute the script"""
         self.machine.run()
@@ -64,6 +59,16 @@ class Nextline:
         """end gracefully"""
         await self.machine.close()
 
+    @property
+    def statement(self) -> str:
+        """The script"""
+        return self.registry.get("statement")
+
+    @property
+    def global_state(self) -> str:
+        """state, e.g., "initialized", "running", "exited", "finished", "closed" """
+        return self.machine.state_name
+
     async def subscribe_global_state(self):
         # wish to be able to write with "yield from" but not possible
         # https://stackoverflow.com/a/59079548/7309855
@@ -71,6 +76,16 @@ class Nextline:
         async for y in agen:
             yield y
 
+    @property
+    def run_no(self):
+        """The current run number"""
+        return self.registry.get("run_no")
+
+    async def subscribe_run_no(self):
+        agen = self.registry.subscribe("run_no")
+        async for y in agen:
+            yield y
+
     async def subscribe_thread_asynctask_ids(self):
         agen = self.registry.subscribe("thread_task_ids")
         async for y in agen:
