@@ -1,10 +1,12 @@
 import asyncio
 import threading
-from typing import Optional, Union
+import itertools
+from typing import Union
 
 from .trace import Trace
 from .utils import Registry, ThreadSafeAsyncioEvent
-from .exec_ import exec_with_trace
+from .call import call_with_trace
+from . import script
 
 SCRIPT_FILE_NAME = "<string>"
 
@@ -31,24 +33,37 @@ class Machine:
             |          | finish()  |
             |          |           |
             |          V           |
-            |    .-------------.   |
-            |----|   Finished  |   |
-            |    '-------------'   |
-            |          |-----------'
-            |          | close()
-            |          V
-            |    .-------------.
-            '----|   Closed    |
-                 '-------------'
+            |    .-------------.   |  close()  .-------------.
+            '----|  Finished   |-------------->|   Closed    |
+                 '-------------'               '-------------'
+
+
+    Parameters
+    ----------
+    statement : str
+        A Python code as a string.
 
     """
 
-    def __init__(self, statement):
-        self.condition_finish = asyncio.Condition()
-        self.condition_close = asyncio.Condition()
+    def __init__(self, statement: str, run_no_start_from=1):
+        self.registry = Registry()
+        self.registry.open_register("statement")
+        self.registry.open_register("state_name")
+        self.registry.open_register("script_file_name")
+        self.registry.open_register("run_no")
+        self.registry.open_register("run_no_count")
+        self.registry.open_register_list("thread_task_ids")
 
-        self.state = Initialized(statement)
-        self.registry = self.state.registry
+        self.registry.register("statement", statement)
+        self.registry.register("script_file_name", SCRIPT_FILE_NAME)
+        self.registry.register(
+            "run_no_count", itertools.count(run_no_start_from).__next__
+        )
+
+        self._state = Initialized(self.registry)
+
+        self._lock_finish = asyncio.Condition()
+        self._lock_close = asyncio.Condition()
 
     def __repr__(self):
         # e.g., "<Machine 'running'>"
@@ -57,11 +72,14 @@ class Machine:
     @property
     def state_name(self) -> str:
         """e.g., "initialized", "running","""
-        return self.state.name
+        try:
+            return self._state.name
+        except BaseException:
+            return "unknown"
 
     def run(self):
         """Enter the running state"""
-        self.state = self.state.run()
+        self._state = self._state.run()
         self._task_exited = asyncio.create_task(self._exited())
 
     async def _exited(self):
@@ -73,31 +91,34 @@ class Machine:
         execution (in another thread) to exit.
         """
 
-        self.state = await self.state.exited()
+        self._state = await self._state.exited()
 
     def send_pdb_command(self, thread_asynctask_id, command):
-        self.state.send_pdb_command(thread_asynctask_id, command)
+        self._state.send_pdb_command(thread_asynctask_id, command)
 
     async def finish(self):
         """Enter the finished state"""
         await self._task_exited
-        async with self.condition_finish:
-            self.state = await self.state.finish()
+        async with self._lock_finish:
+            self._state = await self._state.finish()
 
     def exception(self):
-        return self.state.exception()
+        return self._state.exception()
 
     def result(self):
-        self.state.result()
+        self._state.result()
 
-    def reset(self, statement=None):
+    def reset(self, statement: Union[str, None] = None):
         """Enter the initialized state"""
-        self.state = self.state.reset(statement=statement)
+        if statement:
+            self.registry.register("statement", statement)
+        self._state = self._state.reset()
 
     async def close(self):
         """Enter the closed state"""
-        async with self.condition_close:
-            self.state = await self.state.close()
+        async with self._lock_close:
+            self._state = self._state.close()
+            await self.registry.close()
 
 
 # __________________________________________________________________||
@@ -148,11 +169,11 @@ class State(ObsoleteMixin):
         self.assert_not_obsolete()
         raise StateMethodError(f"Irrelevant operation on the state: {self!r}")
 
-    def reset(self, *_, **__):
+    def reset(self):
         self.assert_not_obsolete()
         raise StateMethodError(f"Irrelevant operation on the state: {self!r}")
 
-    async def close(self):
+    def close(self):
         self.assert_not_obsolete()
         raise StateMethodError(f"Irrelevant operation on the state: {self!r}")
 
@@ -171,60 +192,34 @@ class Initialized(State):
 
     Parameters
     ----------
-    statement : str or None
-        A Python code as a string. None can be given by a state object
-        when the state is reset. If None, the statement will be
-        obtained from the registry.
-    registry : object, optional
-        An instance of Registry. This parameter will be given by a state
-        object when the state is reset.
+    registry : object
+        An instance of Registry.
 
     """
 
     name = "initialized"
 
-    def __init__(
-        self, statement: Union[str, None], registry: Optional[Registry] = None
-    ):
-
-        if registry:
-            self.registry = registry
-        else:
-            self.registry = Registry()
-            self.registry.open_register("statement")
-            self.registry.open_register("state_name")
-            self.registry.open_register("script_file_name")
-            self.registry.open_register_list("thread_task_ids")
-
-        if statement:
-            self.registry.register("statement", statement)
-            self.registry.register("script_file_name", SCRIPT_FILE_NAME)
-        else:
-            statement = self.registry.get("statement")
-
-        if isinstance(statement, str):
-            self._code = compile(statement, SCRIPT_FILE_NAME, "exec")
-        else:
-            self._code = statement
-
+    def __init__(self, registry: Registry):
+        self.registry = registry
+        run_no = self.registry.get("run_no_count")()
+        self.registry.register("run_no", run_no)
         self.registry.register("state_name", self.name)
 
     def run(self):
         self.assert_not_obsolete()
-        running = Running(self.registry, self._code)
+        running = Running(self.registry)
         self.obsolete()
         return running
 
-    def reset(self, statement=None):
+    def reset(self):
         self.assert_not_obsolete()
-        initialized = Initialized(statement=statement, registry=self.registry)
+        initialized = Initialized(registry=self.registry)
         self.obsolete()
         return initialized
 
-    async def close(self):
+    def close(self):
         self.assert_not_obsolete()
         closed = Closed(self.registry)
-        await closed._ainit()
         self.obsolete()
         return closed
 
@@ -236,20 +231,17 @@ class Running(State):
     ----------
     registry : object
         An instance of Registry
-    code : object
-        The code object to be executed.
     """
 
     name = "running"
 
-    def __init__(self, registry, code):
+    def __init__(self, registry):
         self.registry = registry
         self._event = ThreadSafeAsyncioEvent()
 
-        trace = Trace(
-            registry=self.registry,
-            modules_to_trace={exec_with_trace.__module__},
-        )
+        statement = self.registry.get("statement")
+
+        trace = Trace(registry=self.registry)
         self.pdb_ci_registry = trace.pdb_ci_registry
 
         self.registry.register("state_name", self.name)
@@ -257,13 +249,26 @@ class Running(State):
         self.loop = asyncio.get_running_loop()
 
         self._thread = threading.Thread(
-            target=exec_with_trace, args=(code, trace, self._done), daemon=True
+            target=self._run, args=(statement, trace), daemon=True
         )
         self._thread.start()
 
+    def _run(self, code, trace):
+
+        if isinstance(code, str):
+            try:
+                code = compile(code, SCRIPT_FILE_NAME, "exec")
+            except BaseException as e:
+                self._done(None, e)
+                return
+
+        func = script.compose(code)
+
+        call_with_trace(func, trace, self._done)
+
     def _done(self, result=None, exception=None):
         # callback function, to be called from another thread at the
-        # end of exec_with_trace()
+        # end of _run()
 
         if self.loop.is_closed():
             # The exit is not being waited in the main thread, for
@@ -369,7 +374,7 @@ class Finished(State):
     def exception(self):
         """Return the exception of the script execution
 
-        Return None if no execution has been raised.
+        Return None if no exception has been raised.
 
         """
         return self._exception
@@ -396,16 +401,15 @@ class Finished(State):
         self.assert_not_obsolete()
         return self
 
-    def reset(self, statement=None):
+    def reset(self):
         self.assert_not_obsolete()
-        initialized = Initialized(statement=statement, registry=self.registry)
+        initialized = Initialized(registry=self.registry)
         self.obsolete()
         return initialized
 
-    async def close(self):
+    def close(self):
         self.assert_not_obsolete()
         closed = Closed(self.registry)
-        await closed._ainit()
         self.obsolete()
         return closed
 
@@ -423,24 +427,9 @@ class Closed(State):
 
     def __init__(self, registry):
         self.registry = registry
-        self.statement = self.registry.get("statement")
         self.registry.register("state_name", self.name)
 
-    async def _ainit(self):
-        await self.registry.close()  # close here because "await" is
-        # not allowed in __init__()
-
-    def reset(self, statement=None):
-        self.assert_not_obsolete()
-        if statement is None:
-            statement = self.statement
-        initialized = Initialized(statement=statement)
-        self.obsolete()
-        return initialized
-
-    async def close(self):
-        # This method can be called when Nextline.close() is
-        # asynchronously called multiple times.
+    def close(self):
         self.assert_not_obsolete()
         return self
 
