@@ -9,6 +9,12 @@ from .aiterable import aiterable
 
 
 ##__________________________________________________________________||
+def test_sync():
+    # requires a running asyncio event loop
+    with pytest.raises(RuntimeError):
+        _ = Registry()
+
+
 @pytest.mark.asyncio
 async def test_simple_usage():
     obj = Registry()
@@ -23,6 +29,7 @@ async def test_simple_usage():
         obj.open_register(key)
         async for i in aiterable(items):
             obj.register(key, i)
+            assert i == obj.get(key)
         obj.close_register(key)
 
     results = await asyncio.gather(subscribe(), register())
@@ -51,10 +58,15 @@ async def test_simple_usage_list():
 
     async def register():
         obj.open_register_list(key)
+        iexp = 0
         async for i in aiterable(items):
             obj.register_list_item(key, i)
+            assert expected[iexp] == obj.get(key)
+            iexp += 1
         async for i in aiterable(items):
             obj.deregister_list_item(key, i)
+            assert expected[iexp] == obj.get(key)
+            iexp += 1
         obj.close_register(key)
 
     results = await asyncio.gather(subscribe(), register())
@@ -63,6 +75,19 @@ async def test_simple_usage_list():
     await obj.close()
 
 
+@pytest.mark.asyncio
+async def test_get_default():
+    obj = Registry()
+
+    key = "no_such_key"
+    assert obj.get(key) is None
+
+    key = "no_such_key"
+    assert 123 == obj.get(key, 123)
+
+    await obj.close()
+
+
 ##__________________________________________________________________||
 nitems = [0, 1, 2, 50]
 nsubscribers = [0, 1, 2, 70]
